# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                         CREATE BASEMAPS                          #
#                                                                  #
####################################################################

This script creates the final AWI Basemaps in all three projections
and with different color palettes. This happens in multiple steps.

  - the DEM is copied to get one for land and one for sea
  - the coastlines are used as a landmask and cut out of the sea DEM
  - sea and land DEMs are rendered with different color palettes
    to make sure the coastline is prominent inregions with very
    shallow seas where GEBCO might have close to zero values
  - NoData values are set to zero
  - sea and land RGB images are merged into one
  - the shaded map is calculated by combining the colored raster
    with the shading raster

At the end, the raster files are finalized by creating tiles and
pyramids and by adding metadata tags.

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

import shutil

# import AWI Basemap configuration
from config import *


# ==================================================================
#    FUNCTIONS
# ==================================================================

def create_basemap(name, grid, hillshade, color_palette, epsg, output):
    """Create the AWI Basemap RGB grid from various inputs

    Parameters
    ----------
    name : str
        Name of grid version
    grid : str
        Path to input DEM grid
    hillshade : str
        Path to input hillshade grid
    color_palette : str
        Path to input color palette file
    epsg : int
        EPSG code
    output : str
        Path to output RGB grid

    """
    msg = f'Copying {name} DEM...'
    print('====================================================================')
    print()
    print(msg)
    print()
    print(f'{grid} > {TMP_DEM_SEA}')
    print()
    shutil.copy(grid, TMP_DEM_SEA)

    msg = f'Cut land from {name} sea DEM...'
    gshhg_world_name = GSHHG_WORLD.stem
    cmd = (f'gdal_rasterize'
           f' -l {gshhg_world_name}'
           f' -burn {DEM_NODATA}'
           f' {GSHHG_WORLD} {TMP_DEM_SEA}')
    run(msg, cmd)

    msg = f'Rendering {name} sea as colored RGB...'
    cmd = (f'gdaldem'
           f' color-relief'
           f' {TMP_DEM_SEA}'
           f' {color_palette}'
           f' {TMP_RGB_SEA}'
           f' -of GTiff'
           f' -b 1'
           f' -compute_edges'
           f' {GDAL_CREATE_OPTIONS}')
    run(msg, cmd)

    msg = f'Fixing {name} sea Nodata...'
    cmd = (f'gdal_edit'
           f' -a_nodata 0'
           f' {TMP_RGB_SEA}')
    run(msg, cmd)

    msg = f'Rendering {name} land as colored RGB...'
    cmd = (f'gdaldem'
           f' color-relief'
           f' {grid}'
           f' {CP_LAND}'
           f' {TMP_RGB_LAND}'
           f' -of GTiff'
           f' -b 1'
           f' -compute_edges'
           f' {GDAL_CREATE_OPTIONS}')
    run(msg, cmd)

    msg = f'Fixing {name} sea Nodata...'
    cmd = (f'gdal_edit'
           f' -a_nodata 0'
           f' {TMP_RGB_LAND}')
    run(msg, cmd)

    msg = 'Merging sea and land RGB grids...'
    cmd = (f'gdal_merge'
           f' -o {TMP_RGB}'
           f' {TMP_RGB_LAND} {TMP_RGB_SEA}')
    run(msg, cmd)

    # cleanup
    cleanup(TMP_DEM_SEA)
    cleanup(TMP_RGB_LAND)
    cleanup(TMP_RGB_SEA)

    # compile shaded and colored RGB grid
    msg = f'Creating shaded {name} map...'
    cmd = (f'gdal_calc'
           f' --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*254)"'
           f' --format GTiff'
           f' --type Byte'
           f' -A {TMP_RGB}'
           f' --A_band 1'
           f' -B {hillshade}'
           f' --B_band 1'
           f' --allBands=A'
           f' --overwrite'
           f' --outfile {TMP_SHADED}')
    # {GDAL_CREATE_OPTIONS_PY} # TODO
    run(msg, cmd)

    # cleanup
    cleanup(TMP_RGB)

    # finalize grid with GDAL create options
    msg = f'Finalizing {name}...'
    cmd = (f'gdal_translate'
           f' -a_srs EPSG:{epsg}'
           f' -a_nodata {RGB_NODATA}'
           f' -stats'
           f' -of GTiff'
           f' {GDAL_CREATE_OPTIONS_FINAL}'
           f' {TMP_SHADED} {output}')
    run(msg, cmd)

    # cleanup
    cleanup(TMP_SHADED)
    cleanup(Path(f'{TMP_SHADED}.aux.xml'))

    # create internal overview layers
    msg = f'Creating overviews for {name}...'
    cmd = (f'gdaladdo'
           f' -r {OVERVIEW_METHOD}'
           f' --config COMPRESS_OVERVIEW NONE'
           f' --config INTERLEAVE_OVERVIEW PIXEL'
           f' --config BIGTIFF_OVERVIEW IF_SAFER'
           f' --config GDAL_TIFF_OVR_BLOCKSIZE {BLOCKSIZE}'
           f' {output}'
           f' {" ".join(str(x) for x in OVERVIEWS)}')
    run(msg, cmd)

    # set metadata in GeoTIFF tags
    print('====================================================================')
    print()
    print(f'Updating Metadata for {name}...')
    METADATA['NAME'] = name
    METADATA['EPSG'] = epsg
    with rasterio.open(output, 'r+') as ds:
        # update metadata
        ds.update_tags(**METADATA)
    print()


# ==================================================================
#    PROCESS
# ==================================================================

# heading
print()
print('====================================================================')
print(f'{"CREATE BASEMAPS":^68}')

# create the different basemaps with map parameters from config.py
for kwargs in MAPS:
    create_basemap(**kwargs)

# cleanup
cleanup(GSHHG_WORLD)
cleanup(DEM_WORLD)
cleanup(DEM_WORLD_HILLSHADE)
cleanup(DEM_ARCTIC)
cleanup(DEM_ARCTIC_HILLSHADE)
cleanup(DEM_ANTARCTIC)
cleanup(DEM_ANTARCTIC_HILLSHADE)
