# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#             PROCESS ANTARCTIC DIGITAL DATABASE DATA              #
#                                                                  #
####################################################################

This script is processing the Antarctic Digital Database (ADD) input
for the AWI Basemap. Input layers are the ADD coastlines and the ADD
rock outcrops. Both layers are reprojected to a global CRS
(EPSG_WORLD), coastlines are separated by shelf ice and ice sheet,
and all geometries are fixed (with zero buffer).

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

# import AWI Basemap configuration
from config import *

# ==================================================================
#    GLOBAL VARIABLES
# ==================================================================

# projection of ADD data
EPSG_ADD = 3031

# ==================================================================
#    PROCESS
# ==================================================================

# layer names
add_coastline_in_name = ADD_COASTLINE_IN.stem
add_ice_sheet_name = ADD_ICE_SHEET.stem
add_shelf_ice_name = ADD_SHELF_ICE.stem
add_rock_outcrop_in_name = ADD_ROCK_OUTCROP_IN.stem
# DEBUG
# inconsistency with add 7.3 layer names, variable needs to be
# overwritten
# TODO: remove the DEBUG block once the issue is fixed in the ADD layer
add_rock_outcrop_in_name = 'add_rock_outcrop_medium_res_polygon_v7_3'
# /DEBUG
add_rock_outcrop_name = ADD_ROCK_OUTCROP.stem

# heading
print()
print('====================================================================')
print(f'{"ANTARCTIC DIGITAL DATABASE / ADD":^68}')

msg = 'Reprojecting and extracting ice sheet from ADD coastlines...'
cmd = (f'ogr2ogr'
       f' -s_srs EPSG:{EPSG_ADD}'
       f' -t_srs EPSG:{EPSG_WORLD}'
       f' -wrapdateline'
       f' -nln "{add_ice_sheet_name}"'
       f' -nlt  MULTIPOLYGON'
       f' -f GPKG'
       f' -overwrite'
       f' {ADD_ICE_SHEET} {ADD_COASTLINE_IN}'
       f' -dialect SQLite'
       f' -sql "SELECT ST_Buffer(ST_Multi(geom),0.0) from \'{add_coastline_in_name}\' WHERE surface=\'land\'"')
run(msg, cmd)

msg = 'Reprojecting and extracting ice mask from ADD coastlines...'
cmd = (f'ogr2ogr'
       f' -s_srs EPSG:{EPSG_ADD}'
       f' -t_srs EPSG:{EPSG_WORLD}'
       f' -wrapdateline'
       f' -nln "{add_shelf_ice_name}"'
       f' -nlt  MULTIPOLYGON'
       f' -f GPKG -overwrite'
       f' {ADD_SHELF_ICE} {ADD_COASTLINE_IN}'
       f' -dialect SQLite'
       f' -sql "SELECT ST_Buffer(ST_Multi(geom),0.0) from \'{add_coastline_in_name}\' WHERE surface IN (\'ice shelf\',\'rumple\',\'ocean\',\'ice tongue\')"')
run(msg, cmd)

msg = 'Reprojecting ADD rock outcrops...'
cmd = (f'ogr2ogr'
       f' -s_srs EPSG:{EPSG_ADD}'
       f' -t_srs EPSG:{EPSG_WORLD}'
       f' -wrapdateline'
       f' -nln "{add_rock_outcrop_name}"'
       f' -nlt  MULTIPOLYGON'
       f' -f GPKG'
       f' -overwrite'
       f' {ADD_ROCK_OUTCROP} {ADD_ROCK_OUTCROP_IN}'
       f' -dialect SQLite'
       f' -sql "SELECT ST_Buffer(ST_Multi(geom),0.0) from \'{add_rock_outcrop_in_name}\'"')
run(msg, cmd)
