# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#   PROCESS INTERNATIONAL BATHYMETRIC CHART OF THE ARCTIC OCEAN    #
#                                                                  #
####################################################################

This script is processing the International Bathymetric Chart of the
Arctic Ocean (IBCAO) input for the AWI Basemap. The dataset is
reprojected (warped) to the Arctic projection (EPSG_ARCTIC) and
resampled to the target resolution (RESOLUTION_M).

Due to ice shelves there is a very steep drop in the DEM from shelf
ice edge down to the seabed, which creates hillshade artifacts. To
avoid these, the IBCAO grid will be processed twice, once the ice
surface DEM and once the sub ice topography DEM. The main hillshade
is created from the sub ice DEM. Then all data above sea level is
extracted from the ice surface DEM, a hillshade is created for these
areas and the sub ice hillshade is replaced in these areas.

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

# import AWI Basemap configuration
from config import *

# ==================================================================
#    GLOBAL VARIABLES
# ==================================================================

# projection of IBCAO data
EPSG_IBCAO = 3996

SOURCE_NODATA = DEM_NODATA

RESAMPLING = 'cubic'  # 'near'

# ==================================================================
#    PROCESS
# ==================================================================

# heading
print()
print('====================================================================')
print(f'{"INTERNATIONAL BATHYMETRIC CHART OF THE ARCTIC OCEAN / IBCAO":^68}')

# ice surface
msg = 'Warping and resampling IBCAO ice surface to target resolution...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_IBCAO}'
       f' -t_srs EPSG:{EPSG_ARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {IBCAO_IN} {IBCAO_ARCTIC_ICESURFACE}')
run(msg, cmd)

# sub ice
msg = 'Warping and resampling IBCAO sub ice to target resolution...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_IBCAO}'
       f' -t_srs EPSG:{EPSG_ARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {IBCAO_SUBICE_IN} {IBCAO_ARCTIC_SUBICE}')
run(msg, cmd)

# DEBUG
# there is an artifact in IBCAO v5 on the Greenland ice sheet that
# will be clipped out. In a later step the holes will be filled with
# GEBCO.
# TODO: remove DEBUG block once the issue is fixed in IBCAO
clip = PATH_IBCAO / 'clip_ibcao_artifacts.gpkg'
clip_name = clip.stem
msg = 'Fixing issues in IBCAO ice surface raster...'
cmd = (f'gdal_rasterize'
       f' -l {clip_name}'
       f' -burn {DEM_NODATA}'
       f' {clip} {IBCAO_ARCTIC_ICESURFACE}')
run(msg, cmd)
# /DEBUG
