# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#  PROCESS INTERNATIONAL BATHYMETRIC CHART OF THE SOUTHERN OCEAN   #
#                                                                  #
####################################################################

This script is processing the International Bathymetric Chart of the
Southern Ocean (IBCSO) input for the AWI Basemap. The dataset is
reprojected (warped) to the Antarctic projection (EPSG_ANTARCTIC)
and resampled to the target resolution (RESOLUTION_M).

Due to ice shelves there is a very steep drop in the DEM from shelf
ice edge down to the seabed, which creates hillshade artifacts. To
avoid these, the IBCSO grid will be processed twice, once the ice
surface DEM and once the sub ice topography DEM. The main hillshade
is created from the sub ice DEM. Then all data above sea level is
extracted from the ice surface DEM, a hillshade is created for these
areas and the sub ice hillshade is replaced in these areas.

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

# import AWI Basemap configuration
from config import *

# ==================================================================
#    GLOBAL VARIABLES
# ==================================================================

# projection of IBCSO data
EPSG_IBCSO = 9354

SOURCE_NODATA = -32768

RESAMPLING = 'cubic'  # 'near'

# ==================================================================
#    PROCESS
# ==================================================================

# heading
print()
print('====================================================================')
print(f'{"INTERNATIONAL BATHYMETRIC CHART OF THE SOUTHERN OCEAN / IBCSO":^68}')

# ice surface
msg = 'Warping and resampling IBCSO ice surface to target resolution...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_IBCSO}'
       f' -t_srs EPSG:{EPSG_ANTARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {IBCSO_IN} {IBCSO_ANTARCTIC_ICESURFACE}')
run(msg, cmd)

# sub ice
msg = 'Warping and resampling IBCSO sub ice to target resolution...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_IBCSO}'
       f' -t_srs EPSG:{EPSG_ANTARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {IBCSO_SUBICE_IN} {IBCSO_ANTARCTIC_SUBICE}')
run(msg, cmd)
