# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#       PROCESS GENERAL BATHYMETRIC CHART OF THE OCEANS DATA       #
#                                                                  #
####################################################################

This script is processing the General Bathymetric Chart of the
Oceans (GEBCO) input for the AWI Basemap. The global map is
resampled to the target resolution (RESOLUTION_DEG) set in the
configuration. Then the global grid is clipped to the Arctic and
Antarctic extent, reprojected (warped) to the Arctic and Antarctic
projections (EPSG_ARCTIC and EPSG_ANTARCTIC), and resampled to the
target resolution (RESOLUTION_M).

Due to ice shelves there is a very steep drop in the DEM from shelf
ice edge down to the seabed, which creates hillshade artifacts. To
avoid these, the GEBCO grid for World and Antarctic outputs will be
processed twice, once the ice surface DEM and once the sub ice
topography DEM. The main hillshade is created from the sub ice DEM.
Then all data above sea level is extracted from the ice surface DEM,
a hillshade is created for these areas and the sub ice hillshade is
replaced in these areas.

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

import shutil

# import AWI Basemap configuration
from config import *

# ==================================================================
#    GLOBAL VARIABLES
# ==================================================================

SOURCE_NODATA = -32768

RESAMPLING = 'cubic'  # 'near'

# ==================================================================
#    PROCESS
# ==================================================================

# heading
print()
print('====================================================================')
print(f'{"GENERAL BATHYMETRIC CHART OF THE OCEANS / GEBCO":^68}')

# World
# if RESOLUTION_DEG almost matches the GEBCO resolution, then use
# original GEBCO input without resampling
if RESOLUTION_DEG == (1 / 60 / 60 * 15):
    tr = ''
else:
    tr = f' -tr {RESOLUTION_DEG} {RESOLUTION_DEG}'

# ice surface
msg = 'Resampling GEBCO ice surface...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_WORLD}'
       f' -t_srs EPSG:{EPSG_WORLD}'
       f'{tr}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {GEBCO_IN} {GEBCO_WORLD_ICESURFACE_TMP1}')
run(msg, cmd)

msg = 'Copying World DEM...'
print('====================================================================')
print()
print(msg)
print()
print(f'{GEBCO_WORLD_ICESURFACE_TMP1} > {GEBCO_WORLD}')
print()
shutil.copy(GEBCO_WORLD_ICESURFACE_TMP1, GEBCO_WORLD)

msg = 'Extracting World DEM above sea level...'
cmd = (f'gdal_calc'
       f' --calc "(A >= 0) * A"'
       f' --format GTiff'
       f' -A {GEBCO_WORLD_ICESURFACE_TMP1}'
       f' --A_band 1'
       f' --overwrite'
       f' --outfile {GEBCO_WORLD_ICESURFACE_TMP2}'
       f' --NoDataValue=0')
run(msg, cmd)

msg = 'Fixing World DEM above sea level NoData...'
cmd = (f'gdalwarp'
       f' -overwrite'
       f' -srcnodata 0'
       f' -dstnodata {DEM_NODATA}'
       f' {GEBCO_WORLD_ICESURFACE_TMP2}'
       f' {GEBCO_WORLD_ICESURFACE}')
run(msg, cmd)

# sub ice
msg = 'Resampling GEBCO sub ice...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_WORLD}'
       f' -t_srs EPSG:{EPSG_WORLD}'
       f'{tr}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {GEBCO_SUBICE_IN} {GEBCO_WORLD_SUBICE}')
run(msg, cmd)

# Arctic
msg = 'Clipping GEBCO for the Arctic...'
cmd = (f'gdal_translate'
       f' -projwin -180.0 90.0 180.0 {ARCTIC_EXTENT_LAT}'
       f' -a_srs EPSG:{EPSG_WORLD}'
       f' -of GTiff'
       f' {GEBCO_IN} {GEBCO_ARCTIC_CLIP}')
run(msg, cmd)

msg = 'Reprojecting and resampling GEBCO for the Arctic...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_WORLD}'
       f' -t_srs EPSG:{EPSG_ARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {GEBCO_ARCTIC_CLIP} {GEBCO_ARCTIC}')
run(msg, cmd)

# Antarctic
msg = 'Clipping GEBCO for the Antarctic...'
cmd = (f'gdal_translate'
       f' -projwin -180.0 {ANTARCTIC_EXTENT_LAT} 180.0 -90.0'
       f' -a_srs EPSG:{EPSG_WORLD}'
       f' -of GTiff'
       f' {GEBCO_IN} {GEBCO_ANTARCTIC_CLIP}')
run(msg, cmd)

msg = 'Reprojecting and resampling GEBCO for the Antarctic...'
cmd = (f'gdalwarp'
       f' -s_srs EPSG:{EPSG_WORLD}'
       f' -t_srs EPSG:{EPSG_ANTARCTIC}'
       f' -tr {RESOLUTION_M} {RESOLUTION_M}'
       f' -r {RESAMPLING}'
       f' -of GTiff'
       f' -srcnodata {SOURCE_NODATA}'
       f' -dstnodata {DEM_NODATA}'
       # f' {WARP_OPTIONS}' # TODO
       f' {GDAL_CREATE_OPTIONS}'
       f' {GEBCO_ANTARCTIC_CLIP} {GEBCO_ANTARCTIC}')
run(msg, cmd)

# cleanup
cleanup(GEBCO_WORLD_ICESURFACE_TMP1)
cleanup(GEBCO_WORLD_ICESURFACE_TMP2)
cleanup(GEBCO_ARCTIC_CLIP)
cleanup(GEBCO_ANTARCTIC_CLIP)
