# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                           PREPARE DEMs                           #
#                                                                  #
####################################################################

This script prepares the GEBCO, IBCAO and IBCSO raster DEMs to get
the final DEMs (World, Arctic, Antarctic), both sub ice and ice
surface.

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

import shutil

# import AWI Basemap configuration
from config import *

# ==================================================================
#    PROCESS
# ==================================================================

# heading
print()
print('====================================================================')
print(f'{"PREPARE DEMs":^68}')

# World
msg = 'Copying World DEM...'
print('====================================================================')
print()
print(msg)
print()
print(f'{GEBCO_WORLD} > {DEM_WORLD}')
print()
shutil.copy(GEBCO_WORLD, DEM_WORLD)

# Arctic
# ice surface
msg = 'Merging Arctic ice surface DEMs...'
cmd = (f'gdal_merge'
       f' -o {DEM_ARCTIC}'
       f' -n {DEM_NODATA}'
       f' -a_nodata {DEM_NODATA}'
       f' {GEBCO_ARCTIC} {IBCAO_ARCTIC_ICESURFACE}')
run(msg, cmd)

msg = 'Copying Arctic DEM...'
print('====================================================================')
print()
print(msg)
print()
print(f'{DEM_ARCTIC} > {DEM_ARCTIC_ICESURFACE_TMP1}')
print()
shutil.copy(DEM_ARCTIC, DEM_ARCTIC_ICESURFACE_TMP1)

msg = 'Extracting Arctic DEM above sea level...'
cmd = (f'gdal_calc'
       f' --calc "(A >= 0) * A"'
       f' --format GTiff'
       f' -A {DEM_ARCTIC_ICESURFACE_TMP1}'
       f' --A_band 1'
       f' --overwrite'
       f' --outfile {DEM_ARCTIC_ICESURFACE_TMP2}'
       f' --NoDataValue=0')
run(msg, cmd)

msg = 'Fixing Arctic DEM above sea level NoData...'
cmd = (f'gdalwarp'
       f' -overwrite'
       f' -srcnodata 0'
       f' -dstnodata  {DEM_NODATA}'
       f' {DEM_ARCTIC_ICESURFACE_TMP2}'
       f' {DEM_ARCTIC_ICESURFACE}')
run(msg, cmd)

# sub ice
msg = 'Merging Arctic sub ice DEMs...'
cmd = (f'gdal_merge'
       f' -o {DEM_ARCTIC_SUBICE}'
       f' -n {DEM_NODATA}'
       f' -a_nodata {DEM_NODATA}'
       f' {GEBCO_ARCTIC} {IBCAO_ARCTIC_SUBICE}')
run(msg, cmd)

# Antarctic
# ice surface
msg = 'Merging Antarctic ice surface DEMs...'
cmd = (f'gdal_merge'
       f' -o {DEM_ANTARCTIC}'
       f' -n {DEM_NODATA}'
       f' -a_nodata {DEM_NODATA}'
       f' {GEBCO_ANTARCTIC} {IBCSO_ANTARCTIC_ICESURFACE}')
run(msg, cmd)

msg = 'Copying Antarctic DEM...'
print('====================================================================')
print()
print(msg)
print()
print(f'{DEM_ANTARCTIC} > {DEM_ANTARCTIC_ICESURFACE_TMP1}')
print()
shutil.copy(DEM_ANTARCTIC, DEM_ANTARCTIC_ICESURFACE_TMP1)

msg = 'Extracting Antarctic DEM above sea level...'
cmd = (f'gdal_calc'
       f' --calc "(A >= 0) * A"'
       f' --format GTiff'
       f' -A {DEM_ANTARCTIC_ICESURFACE_TMP1}'
       f' --A_band 1'
       f' --overwrite'
       f' --outfile {DEM_ANTARCTIC_ICESURFACE_TMP2}'
       f' --NoDataValue=0')
run(msg, cmd)

msg = 'Fixing Antarctic DEM above sea level NoData...'
cmd = (f'gdalwarp'
       f' -overwrite'
       f' -srcnodata 0'
       f' -dstnodata  {DEM_NODATA}'
       f' {DEM_ANTARCTIC_ICESURFACE_TMP2}'
       f' {DEM_ANTARCTIC_ICESURFACE}')
run(msg, cmd)

# sub ice
msg = 'Merging Antarctic sub ice DEMs...'
cmd = (f'gdal_merge'
       f' -o {DEM_ANTARCTIC_SUBICE}'
       f' -n {DEM_NODATA}'
       f' -a_nodata {DEM_NODATA}'
       f' {GEBCO_ANTARCTIC} {IBCSO_ANTARCTIC_SUBICE}')
run(msg, cmd)

# cleanup
cleanup(GEBCO_ARCTIC)
cleanup(GEBCO_ANTARCTIC)
cleanup(IBCAO_ARCTIC_ICESURFACE)
cleanup(IBCAO_ARCTIC_SUBICE)
cleanup(IBCSO_ANTARCTIC_ICESURFACE)
cleanup(IBCSO_ANTARCTIC_SUBICE)
cleanup(DEM_ARCTIC_ICESURFACE_TMP1)
cleanup(DEM_ARCTIC_ICESURFACE_TMP2)
cleanup(DEM_ANTARCTIC_ICESURFACE_TMP1)
cleanup(DEM_ANTARCTIC_ICESURFACE_TMP2)
