# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                          CONFIGURATION                           #
#                                                                  #
####################################################################

This script holds the configuration for the AWI Basemap creation
process. Adjust the settings below and the other basemap python
scripts will source these settings. The following can be adjusted
here:

- Metadata
- Input folder paths
- Input file paths
- General create options (like resolution, extent, etc.)
- Paths to temporary files
- Map versions (output files, names, color palettes)
- Operational parameters for the commands
- A small set of functions used by the other scripts

"""

# ==================================================================
#    SCRIPT INFO
# ==================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.7'
__date__ = '2024-11-18'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ==================================================================
#    IMPORT
# ==================================================================

import datetime as dt
import os
from pathlib import Path
import pyproj
import sys

from osgeo import gdal
import rasterio

# ==================================================================
#    PROJECTIONS
# ==================================================================

# The EPSG codes for the three map projections only need to be
# changed, if a different projection is required for the Map Viewers

# EPSG code for World Basemap
EPSG_WORLD = 4326  # WGS84 / Geographic
CRS_WORLD = pyproj.CRS.from_epsg(EPSG_WORLD)

# EPSG code for Arctic Basemap
EPSG_ARCTIC = 3995  # WGS 84 / Arctic Polar Stereographic
CRS_ARCTIC = pyproj.CRS.from_epsg(EPSG_ARCTIC)

# EPSG code for Antarctic Basemap
EPSG_ANTARCTIC = 3031  # WGS 84 / Antarctic Polar Stereographic
CRS_ANTARCTIC = pyproj.CRS.from_epsg(EPSG_ANTARCTIC)

# ==================================================================
#    METADATA
# ==================================================================

# metadata fields as convenient text blocks for easy writing
TITLE = 'AWI Basemap'

VERSION = '2024'

ABSTRACT = f'''
The {TITLE} (version {VERSION}) is a global basemap for GIS applications or web map viewers. The map is a rendered and shaded RGB version of the current GEBCO grid with polar grids from IBCAO and IBCSO and ice overlays from ADD, GLIMS and GIMP. It is available in three projections, {CRS_WORLD.to_string()} for the global map as well as {CRS_ARCTIC.to_string()} and {CRS_ANTARCTIC.to_string()} for the polar stereographic versions.
'''.strip()

SOURCES = '''
GEBCO Compilation Group (2024) GEBCO 2024 Grid. https://doi.org/10.5285/1c44ce99-0a0d-5f4f-e063-7086abc0ea0f
Jakobsson, M., Mayer, L.A., Bringensparr, C. et al. The International Bathymetric Chart of the Arctic Ocean Version 4.0. Sci Data 7, 176 (2020). https://doi.org/10.1038/s41597-020-0520-9
Dorschel, Boris; Hehemann, Laura; Viquerat, Sacha; Warnke, Fynn; Dreutter, Simon; Schulze Tenberge, Yvonne et al. (2022): The International Bathymetric Chart of the Southern Ocean Version 2 (IBCSO v2) [dataset]. PANGAEA, https://doi.org/10.1594/PANGAEA.937574
GLIMS Consortium, 2005. GLIMS Glacier Database, Version 1. Boulder Colorado, USA. NASA National Snow and Ice Data Center Distributed Active Archive Center. https://doi.org/10.7265/N5V98602 [2023-05-16]
Howat, I. 2017. MEaSUREs Greenland Ice Mapping Project (GIMP) Land Ice and Ocean Classification Mask, Version 1. [GimpIceMask_90m_2015_v1.2]. Boulder, Colorado USA. NASA National Snow and Ice Data Center Distributed Active Archive Center. https://doi.org/10.5067/B8X58MQBFUPA [2024-09-09]
Howat, I., A. Negrete, and B. Smith. 2014. The Greenland Ice Mapping Project (GIMP) land classification and surface elevation data sets. The Cryosphere, 8. 1509-1518. https://doi.org/10.5194/tc-8-1509-2014
Gerrish, L., Ireland, L., Fretwell, P., & Cooper, P. (2024). Medium resolution vector polygons of the Antarctic coastline (7.9) [Data set]. UK Polar Data Centre, Natural Environment Research Council, UK Research & Innovation. https://doi.org/10.5285/aaec1295-b0a8-4c49-a751-d964c326ce8d
Gerrish, L., Fretwell, P., & Cooper, P. (2020). Medium resolution vector polygons of Antarctic rock outcrop (7.3) [Data set]. UK Polar Data Centre, Natural Environment Research Council, UK Research & Innovation. https://doi.org/10.5285/077e1f04-7068-4327-a4f2-71d863f70064
Wessel, P., and W. H. F. Smith, A Global Self-consistent, Hierarchical, High-resolution Shoreline Database, J. Geophys. Res., 101, 8741-8743, 1996.
'''.strip()

ABSTRACT_WMS = f'''
{ABSTRACT}

Sources:
{SOURCES}
'''.strip()

KEYWORDS = ['Basemap',
            'AWI Basemap',
            'Bathymetry',
            'Topography',
            'GEBCO']

LICENSE = 'CC BY 4.0'

USAGE_FEES = 'None'

CREDITS = 'Simon Dreutter (simon.dreutter@awi.de) - AWI Bathymetry Group'

CREATION_COMMENTS = f'''
Python version {sys.version}
GDAL version {gdal.__version__}
Rasterio version {rasterio.__version__}
AWI Basemap script version {__version__} ({__date__})
grid creation date: {dt.date.today():%Y-%m-%d}
'''.strip()

# ==================================================================

# metadata compiled as dictionary for later script usage
METADATA = {
    'VERSION': VERSION,
    'ABSTRACT': ABSTRACT,
    'SOURCES': SOURCES,
    'KEYWORDS': ','.join(KEYWORDS),
    'LICENSE': LICENSE,
    'USAGE_FEES': USAGE_FEES,
    'CREDITS': CREDITS,
    'CREATION_COMMENTS': CREATION_COMMENTS,
}

# ==================================================================
#    INPUT FOLDERS
# ==================================================================

# These folders are part of the processing infrastructure. Only
# change the variables if the directory names or locations have been
# changed.

# path to data directory
PATH_DATA = Path('..', 'data').resolve()

# path to separate datasets
PATH_ADD = PATH_DATA / 'add'
PATH_GEBCO = PATH_DATA / 'gebco'
PATH_GIMP = PATH_DATA / 'gimp'
PATH_GLIMS = PATH_DATA / 'glims'
PATH_GSHHG = PATH_DATA / 'gshhg'
PATH_IBCAO = PATH_DATA / 'ibcao'
PATH_IBCSO = PATH_DATA / 'ibcso'

# path to result directory
PATH_RESULT = Path('..', 'result').resolve()

# ==================================================================
#    INPUT FILES
# ==================================================================

# These file paths are using the current version of the input files'
# names. If any of the input files are updated in the data
# directory, their filenames need to be updated in this section.

# input filepath to ADD Coastlines
ADD_COASTLINE_IN = PATH_ADD / 'add_coastline_medium_res_polygon_v7_9.gpkg'
# input filepath to ADD Rock Outcrops
ADD_ROCK_OUTCROP_IN = PATH_ADD / 'add_rock_outcrop_medium_res_polygon_v7.3.gpkg'

# input filepath to GEBCO netCDF grids
GEBCO_IN = PATH_GEBCO / 'GEBCO_2024.nc'
GEBCO_SUBICE_IN = PATH_GEBCO / 'GEBCO_2024_sub_ice_topo.nc'

# input filepath to GIMP GeoTIFF
GIMP_IN = PATH_GIMP / 'GimpIceMask_90m_2015_v1.2.tif'

# input filepath to GLIMS Esri Shapefile
GLIMS_IN = PATH_GLIMS / 'glims_polygons.shp'

# input filepath to GSHHG Esri Shapefile
GSHHG_IN = PATH_GSHHG / 'GSHHS_f_L1.shp'

# input filepath to IBCAO GeoTIFF grids
IBCAO_IN = PATH_IBCAO / 'ibcao_v5_2024_ice_400m.tiff'
IBCAO_SUBICE_IN = PATH_IBCAO / 'ibcao_v5_2024_depth_400m.tiff'

# input filepath to IBCSO GeoTIFF grids
IBCSO_IN = PATH_IBCSO / 'IBCSO_v2_ice-surface.tif'
IBCSO_SUBICE_IN = PATH_IBCSO / 'IBCSO_v2_bed.tif'

# ==================================================================
#    GENERAL CREATE OPTIONS
# ==================================================================

# Here are some general options for the resulting grids like
# resolution and shading exaggeration. Adjusting the resolution of
# the output grid will directly affect file sizes. The maximum
# resolution for the World map is the original GEBCO resolution
# (15 arc seconds / 0.00416... degree). For the Polar maps, higher
# resolutions than 500m can be achieved theoretically, but this will
# result in some ugly artifacts (from warping) in the lower
# latitudes.
#
# The best tradeoff between file size, resolution and artifacts is
#   World: 30 arc seconds / 0.0083... degree
#   Polar: 1000 meter
#
# The extent of the Polar maps should be kept to an acceptable
# minium (larger number) to reduce file size and artifacts from
# warping in the lower latitudes.

# Final AWI Basemap (World) resolution
RESOLUTION_DEG = 1 / 60 / 60 * 15  # 15 arc seconds (original GEBCO resolution)
# RESOLUTION_DEG = 1 / 60 / 60 * 30  # 30 arc seconds

# Final AWI Basemap (Arctic and Antarctic) resolution
RESOLUTION_M = 500.0  # approximate pendant to 15 arc seconds
# RESOLUTION_M = 1000.0  # approximate pendant to 30 arc seconds

# Arctic extent down to X degree latitude (positive)
ARCTIC_EXTENT_LAT = 30.0

# Antarctic extent up to X degree latitude (negative)
ANTARCTIC_EXTENT_LAT = -30.0

# Hillshade z factor / vertical exaggeration
Z_FACTOR = 10.0

# ==================================================================
#    TEMPORARY FILES
# ==================================================================

# These are file paths to temporary files that will be created (and
# cleaned up) during the process. The variables care used throughout
# the different scripts.

# GSHHG:
GSHHG_WORLD = PATH_GSHHG / 'gshhg_world.gpkg'

# ==================================================================

# ADD:
ADD_ICE_SHEET = PATH_ADD / 'add_ice_sheet.gpkg'
ADD_SHELF_ICE = PATH_ADD / 'add_shelf_ice.gpkg'
ADD_ROCK_OUTCROP = PATH_ADD / 'add_rock_outcrop.gpkg'

# ==================================================================

# GIMP:
GIMP_RASTER = PATH_GIMP / 'gimp_ice_mask.tif'
GIMP = PATH_GIMP / 'gimp_ice_mask.gpkg'

# ==================================================================

# GLIMS:
GLIMS_WORLD = PATH_GLIMS / 'glims_world.gpkg'
GLIMS_ARCTIC = PATH_GLIMS / 'glims_arctic.gpkg'
GLIMS_ANTARCTIC = PATH_GLIMS / 'glims_antarctic.gpkg'

# ==================================================================

# IBCAO:
IBCAO_ARCTIC_ICESURFACE = PATH_IBCAO / 'ibcao_arctic_icesurface.tif'
IBCAO_ARCTIC_SUBICE = PATH_IBCAO / 'ibcao_arctic_subice.tif'

# ==================================================================

# IBCSO:
IBCSO_ANTARCTIC_ICESURFACE = PATH_IBCSO / 'ibcso_antarctic_icesurface.tif'
IBCSO_ANTARCTIC_SUBICE = PATH_IBCSO / 'ibcso_antarctic_subice.tif'

# ==================================================================

# GEBCO:
# GEBCO World ice surface
GEBCO_WORLD_ICESURFACE_TMP1 = PATH_GEBCO / 'gebco_world_icesurface_tmp1.tif'
GEBCO_WORLD_ICESURFACE_TMP2 = PATH_GEBCO / 'gebco_world_icesurface_tmp2.tif'
GEBCO_WORLD_ICESURFACE = PATH_GEBCO / 'gebco_world_icesurface.tif'
GEBCO_WORLD_ICESURFACE_HILLSHADE = PATH_GEBCO / 'gebco_world_icesurface_hillshade.tif'

# GEBCO World sub ice
GEBCO_WORLD_SUBICE = PATH_GEBCO / 'gebco_world_subice.tif'
GEBCO_WORLD_SUBICE_HILLSHADE = PATH_GEBCO / 'gebco_world_subice_hillshade.tif'

# GEBCO World
GEBCO_WORLD = PATH_GEBCO / 'gebco_world.tif'

# GEBCO Arctic
GEBCO_ARCTIC_CLIP = PATH_GEBCO / 'gebco_arctic_clip.tif'
GEBCO_ARCTIC = PATH_GEBCO / 'gebco_arctic.tif'

# GEBCO Antarctic
GEBCO_ANTARCTIC_CLIP = PATH_GEBCO / 'gebco_antarctic_clip.tif'
GEBCO_ANTARCTIC = PATH_GEBCO / 'gebco_antarctic.tif'

# ==================================================================

# DEMs:
# World
DEM_WORLD = PATH_GEBCO / 'dem_world.tif'
DEM_WORLD_HILLSHADE = PATH_GEBCO / 'dem_world_hillshade.tif'

# Arctic
DEM_ARCTIC_ICESURFACE_TMP1 = PATH_GEBCO / 'dem_arctic_icesurface_tmp1.tif'
DEM_ARCTIC_ICESURFACE_TMP2 = PATH_GEBCO / 'dem_arctic_icesurface_tmp2.tif'
DEM_ARCTIC_ICESURFACE = PATH_GEBCO / 'dem_arctic_icesurface.tif'
DEM_ARCTIC_ICESURFACE_HILLSHADE = PATH_GEBCO / 'dem_arctic_icesurface_hillshade.tif'
DEM_ARCTIC_SUBICE = PATH_GEBCO / 'dem_arctic_subice.tif'
DEM_ARCTIC_SUBICE_HILLSHADE = PATH_GEBCO / 'dem_arctic_subice_hillshade.tif'
DEM_ARCTIC = PATH_GEBCO / 'dem_arctic.tif'
DEM_ARCTIC_HILLSHADE = PATH_GEBCO / 'dem_arctic_hillshade.tif'

# Antarctic
DEM_ANTARCTIC_ICESURFACE_TMP1 = PATH_GEBCO / 'dem_antarctic_icesurface_tmp1.tif'
DEM_ANTARCTIC_ICESURFACE_TMP2 = PATH_GEBCO / 'dem_antarctic_icesurface_tmp2.tif'
DEM_ANTARCTIC_ICESURFACE = PATH_GEBCO / 'dem_antarctic_icesurface.tif'
DEM_ANTARCTIC_ICESURFACE_HILLSHADE = PATH_GEBCO / 'dem_antarctic_icesurface_hillshade.tif'
DEM_ANTARCTIC_SUBICE = PATH_GEBCO / 'dem_antarctic_subice.tif'
DEM_ANTARCTIC_SUBICE_HILLSHADE = PATH_GEBCO / 'dem_antarctic_subice_hillshade.tif'
DEM_ANTARCTIC = PATH_GEBCO / 'dem_antarctic.tif'
DEM_ANTARCTIC_HILLSHADE = PATH_GEBCO / 'dem_antarctic_hillshade.tif'

# ==================================================================

# temporary files for final shading
TMP_DEM_SEA = PATH_GEBCO / 'tmp_dem_sea.tif'
TMP_RGB_LAND = PATH_GEBCO / 'tmp_rgb_land.tif'
TMP_RGB_SEA = PATH_GEBCO / 'tmp_rgb_sea.tif'
TMP_RGB = PATH_GEBCO / 'tmp_rgb.tif'
TMP_SHADED = PATH_GEBCO / 'tmp_shaded.tif'

# ==================================================================
#    MAP VERSIONS
# ==================================================================

# In this section the different map versions are defined, based on
# the World, Arctic and Antarctic files. The version definition
# includes output, name and color palette information.

# ==================================================================

# Final Output:
# blue colors
BASEMAP_WORLD_BLUE = PATH_RESULT / 'AWI_Basemap_World_blue.tif'
BASEMAP_ARCTIC_BLUE = PATH_RESULT / 'AWI_Basemap_Arctic_blue.tif'
BASEMAP_ANTARCTIC_BLUE = PATH_RESULT / 'AWI_Basemap_Antarctic_blue.tif'

# greyblue colors
BASEMAP_WORLD_GREYBLUE = PATH_RESULT / 'AWI_Basemap_World_greyblue.tif'
BASEMAP_ARCTIC_GREYBLUE = PATH_RESULT / 'AWI_Basemap_Arctic_greyblue.tif'
BASEMAP_ANTARCTIC_GREYBLUE = PATH_RESULT / 'AWI_Basemap_Antarctic_greyblue.tif'

# grey colors
BASEMAP_WORLD_GREY = PATH_RESULT / 'AWI_Basemap_World_grey.tif'
BASEMAP_ARCTIC_GREY = PATH_RESULT / 'AWI_Basemap_Arctic_grey.tif'
BASEMAP_ANTARCTIC_GREY = PATH_RESULT / 'AWI_Basemap_Antarctic_grey.tif'

# ==================================================================

# color palettes
PATH_COLORS = Path('..', 'styles', 'color_palettes').resolve()

CP_LAND = PATH_COLORS / 'basemap_land.txt'
CP_BLUE = PATH_COLORS / 'basemap_blue_sea.txt'
CP_GREYBLUE = PATH_COLORS / 'basemap_greyblue_sea.txt'
CP_GREY = PATH_COLORS / 'basemap_grey_sea.txt'

# ==================================================================

# map combinations to be compiled at the end
MAPS = [
    {'name': 'AWI Basemap World blue',
     'grid': DEM_WORLD,
     'hillshade': DEM_WORLD_HILLSHADE,
     'color_palette': CP_BLUE,
     'epsg': EPSG_WORLD,
     'output': BASEMAP_WORLD_BLUE},

    {'name': 'AWI Basemap Arctic blue',
     'grid': DEM_ARCTIC,
     'hillshade': DEM_ARCTIC_HILLSHADE,
     'color_palette': CP_BLUE,
     'epsg': EPSG_ARCTIC,
     'output': BASEMAP_ARCTIC_BLUE},

    {'name': 'AWI Basemap Antarctic blue',
     'grid': DEM_ANTARCTIC,
     'hillshade': DEM_ANTARCTIC_HILLSHADE,
     'color_palette': CP_BLUE,
     'epsg': EPSG_ANTARCTIC,
     'output': BASEMAP_ANTARCTIC_BLUE},

    {'name': 'AWI Basemap World greyblue',
     'grid': DEM_WORLD,
     'hillshade': DEM_WORLD_HILLSHADE,
     'color_palette': CP_GREYBLUE,
     'epsg': EPSG_WORLD,
     'output': BASEMAP_WORLD_GREYBLUE},

    {'name': 'AWI Basemap Arctic greyblue',
     'grid': DEM_ARCTIC,
     'hillshade': DEM_ARCTIC_HILLSHADE,
     'color_palette': CP_GREYBLUE,
     'epsg': EPSG_ARCTIC,
     'output': BASEMAP_ARCTIC_GREYBLUE},

    {'name': 'AWI Basemap Antarctic greyblue',
     'grid': DEM_ANTARCTIC,
     'hillshade': DEM_ANTARCTIC_HILLSHADE,
     'color_palette': CP_GREYBLUE,
     'epsg': EPSG_ANTARCTIC,
     'output': BASEMAP_ANTARCTIC_GREYBLUE},

    {'name': 'AWI Basemap World grey',
     'grid': DEM_WORLD,
     'hillshade': DEM_WORLD_HILLSHADE,
     'color_palette': CP_GREY,
     'epsg': EPSG_WORLD,
     'output': BASEMAP_WORLD_GREY},

    {'name': 'AWI Basemap Arctic grey',
     'grid': DEM_ARCTIC,
     'hillshade': DEM_ARCTIC_HILLSHADE,
     'color_palette': CP_GREY,
     'epsg': EPSG_ARCTIC,
     'output': BASEMAP_ARCTIC_GREY},

    {'name': 'AWI Basemap Antarctic grey',
     'grid': DEM_ANTARCTIC,
     'hillshade': DEM_ANTARCTIC_HILLSHADE,
     'color_palette': CP_GREY,
     'epsg': EPSG_ANTARCTIC,
     'output': BASEMAP_ANTARCTIC_GREY},
]

# additional output
METADATA_FILE = PATH_RESULT / 'AWI_Basemap_Metadata.txt'
ADMIN_LAYER_TOML = PATH_RESULT / 'admin.layer.toml'
ADMIN_SERVICE_TOML = PATH_RESULT / 'admin.service.toml'
OWNER_LAYER_TOML = PATH_RESULT / 'owner.layer.toml'

# ==================================================================
#    OPERATIONAL PARAMETERS
# ==================================================================

# The following settings are operational parameters for the process.
# If the temporary GeoTIFFs created by GDAL shall be created with
# different create options, they can be adjusted below.

# GDAL creation options
# GDAL_CREATE_OPTIONS = '-co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'
# GDAL_CREATE_OPTIONS = '-co COMPRESS=NONE -co BIGTIFF=IF_SAFER'
# GDAL_CREATE_OPTIONS = '-co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=YES'
# GDAL_CREATE_OPTIONS = '-co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=IF_NEEDED'
GDAL_CREATE_OPTIONS = '-co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=IF_SAFER'

# GDAL creation options for python modules
# GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=NONE --co BIGTIFF=IF_NEEDED'
# GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=NONE --co BIGTIFF=IF_SAFER'
# GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=DEFLATE --co PREDICTOR=2 --co ZLEVEL=9 --co BIGTIFF=YES'
# GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=DEFLATE --co PREDICTOR=2 --co ZLEVEL=9 --co BIGTIFF=IF_NEEDED'
GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=DEFLATE --co PREDICTOR=2 --co ZLEVEL=9 --co BIGTIFF=IF_SAFER'

# GDAL warp options
WARP_OPTIONS = '-wo SOURCE_EXTRA=1000 -et 0'

# GDAL creation options for final result
BLOCKSIZE = 512
# GDAL_CREATE_OPTIONS_FINAL = f'-co TILED=YES -co BLOCKXSIZE={BLOCKSIZE} -co BLOCKYSIZE={BLOCKSIZE} -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'
# GDAL_CREATE_OPTIONS_FINAL = f'-co TILED=YES -co BLOCKXSIZE={BLOCKSIZE} -co BLOCKYSIZE={BLOCKSIZE} -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=IF_NEEDED'
# GDAL_CREATE_OPTIONS_FINAL = f'-co TILED=YES -co BLOCKXSIZE={BLOCKSIZE} -co BLOCKYSIZE={BLOCKSIZE} -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=NONE -co BIGTIFF=IF_SAFER'
GDAL_CREATE_OPTIONS_FINAL = f'-co TILED=YES -co BLOCKXSIZE={BLOCKSIZE} -co BLOCKYSIZE={BLOCKSIZE} -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=IF_SAFER'

# Method to create overviews
OVERVIEW_METHOD = 'cubic'

# Overview levels
OVERVIEWS = [2, 4, 8, 16, 32, 64]

# DEM NoData value before RGB rendering
DEM_NODATA = -32767
# RGB Nodata value
RGB_NODATA = 255


# ==================================================================
#    RUN FUNCTION
# ==================================================================

# function to run a shell command and print a message to stdout
def run(msg, cmd):
    print('====================================================================')
    print()
    print(msg)
    print()
    print(cmd)
    print()
    os.system(cmd)
    print()


# ==================================================================
#    CLEANUP
# ==================================================================

# list of temporary files that shall not be deleted
DO_NOT_DELETE = [
    # GSHHG_WORLD,
    # ADD_ICE_SHEET,
    # ADD_SHELF_ICE,
    # ADD_ROCK_OUTCROP,
    # GIMP_RASTER,
    # GIMP,
    # GLIMS_WORLD,
    # GLIMS_ARCTIC,
    # GLIMS_ANTARCTIC,
    # IBCAO_ARCTIC_ICESURFACE,
    # IBCAO_ARCTIC_SUBICE,
    # IBCSO_ANTARCTIC_ICESURFACE,
    # IBCSO_ANTARCTIC_SUBICE,
    # GEBCO_WORLD_ICESURFACE_TMP1,
    # GEBCO_WORLD_ICESURFACE_TMP2,
    # GEBCO_WORLD_ICESURFACE,
    # GEBCO_WORLD_ICESURFACE_HILLSHADE,
    # GEBCO_WORLD_SUBICE,
    # GEBCO_WORLD_SUBICE_HILLSHADE,
    # GEBCO_WORLD,
    # GEBCO_ARCTIC_CLIP,
    # GEBCO_ARCTIC,
    # GEBCO_ANTARCTIC_CLIP,
    # GEBCO_ANTARCTIC,
    # DEM_WORLD,
    # DEM_WORLD_HILLSHADE,
    # DEM_ARCTIC_ICESURFACE_TMP1,
    # DEM_ARCTIC_ICESURFACE_TMP2,
    # DEM_ARCTIC_ICESURFACE,
    # DEM_ARCTIC_ICESURFACE_HILLSHADE,
    # DEM_ARCTIC_SUBICE,
    # DEM_ARCTIC_SUBICE_HILLSHADE,
    # DEM_ARCTIC,
    # DEM_ARCTIC_HILLSHADE,
    # DEM_ANTARCTIC_ICESURFACE_TMP1,
    # DEM_ANTARCTIC_ICESURFACE_TMP2,
    # DEM_ANTARCTIC_ICESURFACE,
    # DEM_ANTARCTIC_ICESURFACE_HILLSHADE,
    # DEM_ANTARCTIC_SUBICE,
    # DEM_ANTARCTIC_SUBICE_HILLSHADE,
    # DEM_ANTARCTIC,
    # DEM_ANTARCTIC_HILLSHADE,
]


# function to clean up temporary files if they are not in the
# DO_NOT_DELETE list
def cleanup(dataset):
    if isinstance(dataset, str):
        dataset = Path(dataset)
    if dataset not in DO_NOT_DELETE:
        if dataset.is_file():
            dataset.unlink()
